/**
 * 
 */
package de.digisalt.dsnesds.core.wizards.create.file;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

/**
 * The superclass for all new file wizards. Any subclass should implement one of
 * the three interfaces: IByteStreamInput, IResourceStreamInput or
 * IStringStreamInput which will be used to return the initial content of the
 * new file created. Please take a look at the interfaces to learn how each of
 * them works
 * 
 * @author flo
 */
public abstract class AbstractNewFileWizard extends Wizard implements
		INewWizard
{
	/**
	 * The wizards configuration pojo
	 */
	private NewFileWizardConfigBean				cfg	= null;
	/**
	 * The selection from which the wizard was created
	 */
	protected IStructuredSelection				selection;
	/**
	 * The workbench reference from which the wizard was created
	 */
	protected IWorkbench									workbench;
	/**
	 * The New File creation page used
	 */
	protected AbstractNewFileCreationPage	page;

	/**
	 * The standard constructor configuring the wizard
	 */
	public AbstractNewFileWizard()
	{
		this.cfg = configure();
		init();
	}

	/**
	 * Will be called by the constructor to configure the Wizard
	 */
	private void init()
	{
		setWindowTitle(this.cfg.getTitle());
		// TODO: add more configuration here
	}

	/**
	 * This method should be overwritten by the implementing Wizard. It should
	 * return a configuration pojo for the Wizards implementation.
	 * 
	 * @return the NewFileWizardConfig pojo to be used
	 */
	protected abstract NewFileWizardConfigBean configure();

	@Override
	public void addPages()
	{
		this.page = new AbstractNewFileCreationPage(this.cfg.getTitle(),
				this.selection);
		configureFirstPage();
		addPage(this.page);
		addAdditionalPages();
	}

	/**
	 * This method initializes the first page of the new file creation wizard
	 * using the configbean
	 */
	private void configureFirstPage()
	{
		this.page.setTitle("Select where to create the new "
				+ this.cfg.getContent());
		this.page.setImageDescriptor(this.cfg.getTitleImage());
		this.page.setFileExtension(cfg.getExtension());
	}

	/**
	 * This method should be overwritten by the Wizard extending the
	 * AbstractNewFileWizard. It is there to add more than the standard new file
	 * creation page via addPage(userPage) so the user can create a customized
	 * wizard as he likes.
	 */
	public abstract void addAdditionalPages();

	@Override
	public boolean performFinish()
	{
		IFile file = this.page.createNewFile();
		if (file != null)
		{
			// attach IFile properties (metadata for the IDE)
			try
			{
				attachMetadata(file);
			}
			catch (CoreException e)
			{
				// if some error occurred adding the metadata don't finish successfully
				e.printStackTrace();
				return false;
			}

			// TODO: open file in editor directly
			return true;
		}
		return false;
	}

	private void attachMetadata(IFile file) throws CoreException
	{
		MetadataObject metadata = getMetadata();
		if (metadata != null && metadata.containsMetadata())
		{
			for (String key : metadata.getKeys())
			{
				file.setPersistentProperty(new QualifiedName(metadata.getNamespace(),
						key), metadata.get(key));
			}
		}
	}

	protected abstract MetadataObject getMetadata();

	@Override
	public void init(final IWorkbench workbench,
			final IStructuredSelection selection)
	{
		this.workbench = workbench;
		this.selection = selection;
	}
}
